@@ -34,9 +34,12 @@ class Model:
             raise RuntimeError('cannot use --show_pruned with --only_show_pruned')
         params = {'source': self.options.source,
                   'target': self.options.target,
-                  'model': self.options.model}
+                  'model': self.options.model,
+                  'tags': self.options.tags}
         if self.options.scores is not None:
             params['scores'] = ",".join(self.options.scores)
+        elif self.options.avg_scores:
+            params['scores'] = ''
         if self.options.count:
             r = requests.get(os.path.join(self.options.url, "model/lp/list"), auth=self.auth)
             if r.status_code != 200:
@@ -57,24 +60,26 @@ class Model:
             result = []
             metrics = Counter()
             for item in response:
-                if self.options.model and self.options.model not in item['model']:
-                    continue
                 if self.options.show_pruned or (self.options.show_pruned is False and
                                                 self.options.only_show_pruned is False and item["pruned"] is False) or\
                         (self.options.only_show_pruned and item["pruned"] is True):
-                    if self.options.scores is not None:
+                    if self.options.scores is not None or self.options.avg_scores:
                         new_scores = {}
+                        total_score = 0
                         for p, v in six.iteritems(item['scores']):
-                            p = os.path.basename(p)
                             if not self.is_json:
                                 if isinstance(v, float):
                                     v = {'BLEU': v}
+                                elif isinstance(v, dict) and "score" in v:
+                                    v = v['score']
                                 for m in v:
                                     metrics[m] += 1
                                 v = v.get(self.options.metric)
                             if v is not None:
                                 new_scores[p] = v
+                                total_score += v
                         item['scores'] = new_scores
+                        item['avg_score'] = total_score/len(new_scores) if new_scores else 0
                     result.append(item)
             if not self.is_json:
                 scorenames = {}
@@ -156,7 +161,7 @@ class Model:
                         root.append(k)
                 max_depth = utils.tree_depth(0, root, idx_result)
                 model_maxsize = max_depth + 42
-                scorenames_key = sorted(scorenames.keys())
+                scorenames_key = [] if self.options.scores is None else sorted(scorenames.keys())
                 scoretable = []
                 scorecols = []
                 for i in range(len(scorenames_key)):
@@ -166,7 +171,7 @@ class Model:
                 if self.options.quiet:
                     res = []
                     utils.tree_display(res, 0, root, idx_result, model_maxsize,
-                                       scorenames_key, bestscores, self.options.skip_noscores,
+                                       scorenames_key, bestscores, self.options.avg_scores, self.options.skip_noscores,
                                        self.options.has_noscores, self.options.show_owner,
                                        self.options.show_pruned, self.options.only_show_pruned,
                                        self.options.quiet)
@@ -176,10 +181,12 @@ class Model:
                         header.append("Owner")
                     if self.options.show_pruned:
                         header.append("Pruned")
+                    if self.options.avg_scores:
+                        header.append("Average scores")
                     res1 = PrettyTable(header + scorecols)
                     res1.align["Model ID"] = "l"
                     utils.tree_display(res1, 0, root, idx_result, model_maxsize,
-                                       scorenames_key, bestscores, self.options.skip_noscores,
+                                       scorenames_key, bestscores, self.options.avg_scores, self.options.skip_noscores,
                                        self.options.has_noscores, self.options.show_owner,
                                        self.options.show_pruned, self.options.only_show_pruned,
                                        self.options.quiet)
@@ -216,7 +223,7 @@ class Model:
         res = "ok new tags \"%s\" attached to model %s" % (",".join(self.options.tags),
                                                            self.options.model)
         return res
-    
+
     def tagdel(self):
         taglist = []
         for tag in self.options.tags:
@@ -232,7 +239,7 @@ class Model:
         res = "ok tags \"%s\" are removed from model %s" % (",".join(self.options.tags),
                                                             self.options.model)
         return res
-    
+
     def share(self):
         data = {
             'visibility': 'share',
@@ -245,7 +252,7 @@ class Model:
             raise RuntimeError('incorrect result from \'model/share\' service: %s' % r.text)
         res = 'ok model %s shared with entity %s' % (self.options.model, self.options.entity_code)
         return res
-    
+
     def removeshare(self):
         data = {
             'visibility': 'share',
@@ -260,7 +267,7 @@ class Model:
         res = 'ok share visibility removed on model %s for entity %s' % \
               (self.options.model, self.options.entity_code)
         return res
-    
+
     def open(self):
         data = {
             'visibility': 'open',
@@ -274,7 +281,7 @@ class Model:
             raise RuntimeError('incorrect result from \'model/open\' service: %s' % r.text)
         res = 'ok model %s opened with entity %s' % (self.options.model, self.options.entity_code)
         return res
-    
+
     def removeopen(self):
         data = {
             'visibility': 'open',
@@ -289,9 +296,39 @@ class Model:
         res = 'ok open visibility removed on model %s for entity %s' % \
               (self.options.model, self.options.entity_code)
         return res
-    
+
     def get(self):
-        with open(os.path.expanduser(self.options.output), 'w+') \
+        if not self.options.file:
+            if self.options.output:
+                if not os.path.isdir(self.options.output):
+                    raise Exception("Output is not a existing directory: %s" % self.options.output)
+                output_path = self.options.output
+            else:
+                output_path = os.getcwd()
+            r = requests.get(os.path.join(self.options.url, "model/listfiles", self.options.model),
+                             auth=self.auth)
+            if r.status_code != 200:
+                raise RuntimeError('incorrect result from \'model/listfiles\' service: %s' % r.text)
+            list_files = [f.split('/')[1] for f in list(r.json().keys())]
+            if not list_files:
+                raise Exception("Model does not exist any files: %s" % self.options.model)
+            try:
+                os.makedirs(os.path.join(output_path, self.options.model))
+            except FileExistsError:
+                pass
+            for file in list_files:
+                if file != '':
+                    g = requests.get(os.path.join(self.options.url, "model/getfile/",
+                                                  self.options.model, file),
+                                     params={'is_compressed': True}, auth=self.auth)
+                    if g.status_code != 200:
+                        raise RuntimeError('incorrect result from \'model/getfile\' service: %s' % g.text)
+                    with open(os.path.expanduser(os.path.join(output_path, self.options.model, file)),
+                              'wb+') as output:
+                        output.write(gzip.GzipFile('', 'r', 0, BytesIO(g.content)).read())
+            sys.exit(0)
+
+        with open(os.path.expanduser(self.options.output), 'wb+') \
                 if self.options.output else sys.stdout as output:
             r = requests.get(os.path.join(self.options.url, "model/getfile/",
                                           self.options.model, self.options.file),
@@ -304,12 +341,13 @@ class Model:
             else:
                 utils.write_to_stdout(gzip.GzipFile('', 'r', 0, BytesIO(r.content)).read())
             sys.exit(0)
-            
+
     def delete(self):
         allres = []
         for m in self.options.models:
             if self.options.dryrun or not self.options.force:
-                params = {'recursive': self.options.recursive, 'dryrun': True}
+                params = {'recursive': self.options.recursive, 'dryrun': True,
+                          'include_released': self.options.include_released}
                 r = requests.get(os.path.join(self.options.url,
                                               "model/delete/%s/%s/%s" % (self.options.source,
                                                                          self.options.target,
@@ -317,18 +355,28 @@ class Model:
                                  params=params, auth=self.auth)
                 if r.status_code == 200:
                     mres = r.json()
+                    if not mres:
+                        launcher.LOGGER.info(
+                            'Cannot find any models that are allowed to be deleted. '
+                            + 'If you want to delete release models, use "include_released" option.')
+                        continue
                 else:
-                    launcher.LOGGER.error('cannot remove %s (%s)' % (m, r.text))
+                    launcher.LOGGER.error('Cannot remove %s (%s)' % (m, r.text))
                     continue
-                launcher.LOGGER.info('-- %sremoving %s and %d '
-                                     'childrens:\n\t%s' % (self.options.dryrun and "not " or "",
-                                                           m, len(mres) - 1, "\n\t".join(mres)))
+                if m not in mres:
+                    launcher.LOGGER.info('%s %d child(ren) of the model %s:\n\t%s' % (
+                        self.options.dryrun and 'Without "--dryrun" option, will remove' or 'Removing', len(mres), m,
+                        "\n\t".join(mres)))
+                else:
+                    launcher.LOGGER.info('%s %s and %d child(ren):\n\t%s' % (
+                        self.options.dryrun and 'Without "--dryrun" option, will remove' or 'Removing', m,
+                        len(mres) - 1, "\n\t".join(mres)))
             confirm = self.options.force
             if self.options.dryrun:
                 continue
             confirm = confirm or launcher.confirm()
             if confirm:
-                params = {'recursive': self.options.recursive}
+                params = {'recursive': self.options.recursive, 'include_released': self.options.include_released}
                 r = requests.get(os.path.join(self.options.url,
                                               "model/delete/%s/%s/%s" % (self.options.source,
                                                                          self.options.target,
@@ -339,12 +387,12 @@ class Model:
                     launcher.LOGGER.info('  => %d models removed: %s' % (len(mres), " ".join(mres)))
                     allres += mres
                 else:
-                    launcher.LOGGER.error('cannot remove %s (%s)' % (m, r.text))
+                    launcher.LOGGER.error('Cannot remove %s (%s)' % (m, r.text))
             else:
                 launcher.LOGGER.info("  ... skipping")
         res = "Total %d models removed" % len(allres)
         return res
-    
+
     def add(self):
         if not os.path.exists(self.options.file):
             raise RuntimeError('file `%s` does not exists' % self.options.file)
@@ -381,7 +429,7 @@ class Model:
             raise RuntimeError('incorrect result from \'model/add\' service: %s' % r.text)
         res = r.json()
         return res
-    
+
     def detail(self):
         res = utils.show_model_files_list(url=self.options.url, auth=self.auth,
                                           model_name=self.options.model,
