@@ -2,7 +2,6 @@
 # coding: utf-8
 
 import client.launcher as launcher
-import argparse
 
 
 def define_option():
@@ -19,6 +18,7 @@ def define_option():
     launcher.subparsers_map["role"] = launcher.subparsers.add_parser('role')
     launcher.subparsers_map["group"] = launcher.subparsers.add_parser('group')
     launcher.subparsers_map["share"] = launcher.subparsers.add_parser('share')
+    launcher.subparsers_map["monitoring"] = launcher.subparsers.add_parser('monitoring')
 
     subparsers_auth = launcher.subparsers_map["auth"].add_subparsers(help='sub-command help',
                                                                      dest='subcmd')
@@ -69,11 +69,14 @@ def define_option():
     launcher.shortcut_map["lm"] = ["model", "list"]
     parser_list_models.add_argument('-s', '--source', help='source language prefix')
     parser_list_models.add_argument('-t', '--target', help='target language prefix')
-    parser_list_models.add_argument('-m', '--model', help='beginning pattern on model name')
+    parser_list_models.add_argument('-m', '--model', nargs='*', help='beginning pattern on model name')
+    parser_list_models.add_argument('--tags', nargs="*", help='model tags')
     parser_list_models.add_argument('--skip_noscores', action='store_true',
-                                                     help='skip models without scores')
+                                    help='skip models without scores')
     parser_list_models.add_argument('--has_noscores', action='store_true',
                                     help='only display models with scores missing')
+    parser_list_models.add_argument('--avg_scores', action='store_true',
+                                    help='display model average scores')
     parser_list_models.add_argument('--quiet', '-q', action='store_true',
                                     help='only display matching model ids')
     parser_list_models.add_argument('--show_owner', '-so', action='store_true',
@@ -86,6 +89,10 @@ def define_option():
                                     help='testset patterns to display along with the model')
     parser_list_models.add_argument('--metric', '-M', default='BLEU',
                                     help='metric to display (default BLEU)')
+    parser_list_models.add_argument('--show_pruned', action='store_true',
+                                    help='display pruned and not pruned models')
+    parser_list_models.add_argument('--only_show_pruned', action='store_true',
+                                    help='only display pruned models')
 
     parser_get_model = subparsers_model.add_parser('get', help='download the model')
     parser_get_model.add_argument('model', help='model')
@@ -104,6 +111,8 @@ def define_option():
     parser_delete_models.add_argument('-t', '--target', help='target language', required=True)
     parser_delete_models.add_argument('--recursive', action='store_true',
                                       help='recursive deletion of each model and its descendant')
+    parser_delete_models.add_argument('--include_released', action='store_true',
+                                      help='delete models, including released models')
     parser_delete_models.add_argument('-f', '--force', action='store_true',
                                       help='do not ask confirmation of the deletion')
     parser_delete_models.add_argument('-d', '--dryrun', action='store_true',
@@ -153,8 +162,8 @@ def define_option():
     parser_open_model.add_argument('-e', '--entity_code', help='2-letters entity code', required=True)
 
     parser_removeopen_model = subparsers_model.add_parser('removeopen',
-                                                           help='remove the open visibility '
-                                                                'of a model for an entity')
+                                                          help='remove the open visibility '
+                                                               'of a model for an entity')
     launcher.shortcut_map["mro"] = ["model", "removeopen"]
     parser_removeopen_model.add_argument('-m', '--model', help='model', required=True)
     parser_removeopen_model.add_argument('-e', '--entity_code', help='2-letters entity code', required=True)
@@ -268,6 +277,8 @@ def define_option():
                                      help="service name")
     parser_change_tasks.add_argument('-g', '--gpus',
                                      help="number of gpus", type=int)
+    parser_change_tasks.add_argument('-c', '--cpus',
+                                     help="number of cpus", type=int)
     parser_change_tasks.add_argument('task_ids', nargs='*',
                                      help="task identifiers")
 
