@@ -32,9 +32,9 @@ def get_version():
 
 class VersionAction(argparse.Action):
     def __init__(self, nargs=0, **kw):
-        super(VersionAction, self).__init__(nargs=nargs, **kw)
+        super().__init__(nargs=nargs, **kw)
 
-    def __call__(self, parser, namespace, values, option_string=None):
+    def __call__(self, current_parser, namespace, values, option_string=None):
         print("Client version: %s" % VERSION)
         r = requests.get(os.path.join(os.getenv('LAUNCHER_URL'), "version"))
         if r.status_code != 200:
@@ -49,7 +49,7 @@ LOGGER = None
 
 
 def getjson(config):
-    if config is None or type(config) == dict:
+    if config is None or isinstance(config, dict):
         return None
     if not config.startswith('@'):
         return json.loads(config)
@@ -75,7 +75,7 @@ def find_files_parameters(v, files):
             LOGGER.info('transferring local file: %s -> ${TMP_DIR}/%s', f[0], f[1])
         return "${TMP_DIR}/%s" % global_basename
     if isinstance(v, list):
-        for idx, item in enumerate(v):
+        for idx, _ in enumerate(v):
             v[idx] = find_files_parameters(v[idx], files)
     if isinstance(v, dict):
         for k in v:
@@ -97,15 +97,15 @@ def confirm(prompt=None, resp=False):
         prompt = '%s [%s]|%s: ' % (prompt, 'n', 'y')
 
     while True:
-        ans = input(prompt)
+        ans = six.moves.input(prompt)
         if not ans:
             return resp
         if ans not in ['y', 'Y', 'n', 'N']:
             print('please enter y or n.')
             continue
-        if ans == 'y' or ans == 'Y':
+        if ans in ['y', 'Y']:
             return True
-        if ans == 'n' or ans == 'N':
+        if ans in ['n', 'N']:
             return False
 
 
@@ -123,8 +123,8 @@ subparsers = parser.add_subparsers(help='command help', dest='cmd')
 subparsers.required = True
 
 subparsers_map = {
-    "service": subparsers.add_parser(u'service'),
-    "task": subparsers.add_parser(u'task'),
+    "service": subparsers.add_parser('service'),
+    "task": subparsers.add_parser('task'),
 }
 shortcut_map = {}
 
@@ -159,7 +159,7 @@ exec_arguments = [
                          "help": 'options selected to run the service'}],
     ['-r', '--resource',
      {"help": "alternatively to `options`, resource name to use"}],
-    ['-g', '--gpus', {"type": int, "default": 1, "help": 'number of gpus'}],
+    ['-g', '--gpus', {"type": int, "help": 'number of gpus'}],
     ['-c', '--cpus', {"type": int,
                       "help": 'number of cpus - if not provided, will be obtained '
                               'from pool config'}],
@@ -187,6 +187,8 @@ parser_exec = subparsers.add_parser('exec', help='execute a generic docker-utili
 for arg in exec_arguments:
     parser_exec.add_argument(*arg[:-1], **arg[-1])
 parser_exec.add_argument('docker_command', type=str, nargs='*', help='Docker command')
+parser_exec.add_argument('-e', '--entity_owner', help='entity owner')
+parser_exec.add_argument('--dependency', help='dependency task (will launch after this task is terminated)')
 
 parser_launch = subparsers.add_parser('launch',
                                       help='launch a task on the service associated'
@@ -208,10 +210,18 @@ parser_launch.add_argument('-o', '--options', default='{}',
                            help='options selected to run the service')
 parser_launch.add_argument('-r', '--resource',
                            help="alternatively to `options`, resource name to use")
-parser_launch.add_argument('-g', '--gpus', type=int, default=1,
-                           help='number of gpus')
+parser_launch.add_argument('-g', '--gpus', type=int, help='number of gpus')
 parser_launch.add_argument('-c', '--cpus', type=int, help='number of cpus - if not provided, '
                                                           'will be obtained from pool config')
+parser_launch.add_argument('-c_prepr', '--cpus_prepr', type=int,
+                           help='number of cpus for preprocessor task - if not provided, '
+                                'will be obtained from pool config')
+parser_launch.add_argument('-c_train', '--cpus_train', type=int,
+                           help='number of cpus for training task - if not provided, '
+                                'will be obtained from pool config')
+parser_launch.add_argument('-c_trans', '--cpus_trans', type=int,
+                           help='number of cpus for translation task - if not provided, '
+                                'will be obtained from pool config')
 parser_launch.add_argument('-w', '--wait_after_launch', default=2, type=int,
                            help='if not 0, wait for this number of seconds after launch '
                                 'to check that launch is ok - by default wait for 2 seconds')
@@ -236,6 +246,8 @@ parser_launch.add_argument('--nochainprepr', action='store_true',
 parser_launch.add_argument('--notransasrelease', action='store_true',
                            help='don\'t run translate as release (image >= 1.8.0)')
 parser_launch.add_argument('docker_command', type=str, nargs='*', help='Docker command')
+parser_launch.add_argument('--tags', nargs='*', help='model tags')
+parser_launch.add_argument('--dependency', help='dependency task (will launch after this task is terminated)')
 
 parser_list_tasks = subparsers_tasks.add_parser('list',
                                                 help='{lt} list tasks matching prefix pattern')
@@ -286,27 +298,27 @@ def _format_message(msg, length=40):
     return msg
 
 
-def _parse_local_filename(arg, files):
-    if os.path.isabs(arg):
-        if not os.path.exists(arg):
-            raise ValueError("file '%s' does not exist" % arg)
-    elif arg.find('/') != -1 and arg.find(':') == -1:
-        if not os.path.exists(arg):
+def _parse_local_filename(argument, files):
+    if os.path.isabs(argument):
+        if not os.path.exists(argument):
+            raise ValueError("file '%s' does not exist" % argument)
+    elif argument.find('/') != -1 and argument.find(':') == -1:
+        if not os.path.exists(argument):
             LOGGER.warning("parameter %s could be a filename but does not exists, "
-                           "considering it is not" % arg)
-            return arg
-        LOGGER.warning("parameter %s could be a filename and exists, considering it is" % arg)
+                           "considering it is not", argument)
+            return argument
+        LOGGER.warning("parameter %s could be a filename and exists, considering it is", argument)
     else:
-        return arg
+        return argument
 
-    LOGGER.debug("considering %s is a file" % arg)
+    LOGGER.debug("considering %s is a file", argument)
 
-    basename = os.path.basename(arg)
+    basename = os.path.basename(argument)
     if basename not in files:
-        files[basename] = (basename, open(arg, 'rb'))
-    arg = "${TMP_DIR}/%s" % basename
+        files[basename] = (basename, open(argument, 'rb'))
+    argument = "${TMP_DIR}/%s" % basename
 
-    return arg
+    return argument
 
 
 def argparse_preprocess():
@@ -316,7 +328,7 @@ def argparse_preprocess():
             skip = False
             continue
         if "--display".startswith(v) or v == "-d" or "--url".startswith(v) or v == "-u" or \
-                "--log-level".startswith(v) or v == "-l":
+                "--log-level".startswith(v) or v == "-l":  # pylint: disable=too-many-boolean-expressions
             skip = True
         elif v == "-v" or "--version".startswith(v):
             continue
@@ -326,7 +338,7 @@ def argparse_preprocess():
             return
 
 
-def process_request(service_list, cmd, subcmd, is_json, args, auth=None):
+def process_request(s_list, cmd, subcmd, is_json, args, auth=None):
     res = None
     result = None
     if cmd == "service" and subcmd == "list":
@@ -365,7 +377,7 @@ def process_request(service_list, cmd, subcmd, is_json, args, auth=None):
                                      '(%d,%d)' % tuple(result[k]['detail'][r]['capacity']),
                                      'yes' if result[k]['detail'][r]['busy'] else '',
                                      err])
-            if len(busymsg):
+            if busymsg:
                 res += "\n" + "\n".join(busymsg)
     elif cmd == "task" and subcmd == "list":
         r = requests.get(os.path.join(args.url, "task/list", args.prefix + '*'), auth=auth,
@@ -380,7 +392,9 @@ def process_request(service_list, cmd, subcmd, is_json, args, auth=None):
             for k in result:
                 res.append(k["task_id"])
         elif not is_json:
-            headers= ["Task ID", "Service", "Resource", "Priority", "Launch Date", "Image", "Status", "Message", "owner"]
+            headers = ["Task ID", "Service", "Resource",
+                       "Priority", "Launch Date", "Image",
+                       "Status", "Message", "owner"]
             if args.parent:
                 headers.append("parent")
 
@@ -406,14 +420,14 @@ def process_request(service_list, cmd, subcmd, is_json, args, auth=None):
         else:
             res = r.json()
     elif cmd == "service" and subcmd == "describe":
-        if args.service not in service_list:
+        if args.service not in s_list:
             raise ValueError("ERROR: service '%s' not defined" % args.service)
         r = requests.get(os.path.join(args.url, "service/describe", args.service), auth=auth)
         if r.status_code != 200:
             raise RuntimeError('incorrect result from \'service/describe\' service: %s' % r.text)
         res = r.json()
     elif cmd == "service" and subcmd == "check":
-        if args.service not in service_list:
+        if args.service not in s_list:
             raise ValueError("ERROR: service '%s' not defined" % args.service)
         if args.options == '{}' and args.resource is not None:
             options = {"server": args.resource}
@@ -442,7 +456,7 @@ def process_request(service_list, cmd, subcmd, is_json, args, auth=None):
             args.docker_tag = m.group(5)
         args.docker_image = m.group(3)
 
-        if args.service not in service_list:
+        if args.service not in s_list:
             raise ValueError("service '%s' not defined" % args.service)
 
         # for multi-part file sending
@@ -450,7 +464,6 @@ def process_request(service_list, cmd, subcmd, is_json, args, auth=None):
         docker_command = []
 
         for c in args.docker_command:
-            orgc = c
             if c.startswith("@"):
                 with open(c[1:], "rt") as f:
                     c = f.read()
@@ -462,9 +475,15 @@ def process_request(service_list, cmd, subcmd, is_json, args, auth=None):
                 c = json.dumps(cjson)
             docker_command.append(c)
 
-        if args.service not in service_list:
+        if args.service not in s_list:
             raise ValueError("ERROR: service '%s' not defined" % args.service)
 
+        if args.gpus is None:
+            if "train" in args.docker_command:
+                args.gpus = 1
+            else:
+                args.gpus = 0
+
         if args.gpus < 0:
             raise ValueError("ERROR: ngpus must be >= 0")
 
@@ -484,7 +503,7 @@ def process_request(service_list, cmd, subcmd, is_json, args, auth=None):
             "trainer_id": args.trainer_id,
             "options": options,
             "ngpus": args.gpus,
-            "ncpus": args.cpus
+            "ncpus": args.cpus,
         }
 
         if cmd == "exec":
@@ -493,8 +512,14 @@ def process_request(service_list, cmd, subcmd, is_json, args, auth=None):
             content["name"] = args.name
         if args.priority:
             content["priority"] = args.priority
+        if args.dependency:
+            content["dependency"] = args.dependency
 
         if cmd == "task" and subcmd == "launch":
+            content["ncpus_prepr"] = args.cpus_prepr
+            content["ncpus_train"] = args.cpus_train
+            content["ncpus_trans"] = args.cpus_trans
+
             if args.iterations:
                 content["iterations"] = args.iterations
             if args.nochainprepr:
@@ -516,6 +541,15 @@ def process_request(service_list, cmd, subcmd, is_json, args, auth=None):
                 content["totuminer"] = [(_parse_local_filename(i, files),
                                          o) for (i, o) in ARGS.totuminer]
 
+            if args.tags:
+                content['tags'] = []
+                for tag in args.tags:
+                    tmp = {'tag': tag}
+                    if ':' in tag:
+                        entity, tag_name = tag.split(':')
+                        tmp = {'entity': entity, 'tag': tag_name}
+                    content['tags'].append(tmp)
+
         LOGGER.debug("sending request: %s", json.dumps(content))
 
         launch_url = os.path.join(args.url, "task/launch", args.service)
@@ -578,10 +612,11 @@ def process_request(service_list, cmd, subcmd, is_json, args, auth=None):
                     date = datetime.fromtimestamp(math.ceil(current)).isoformat(' ')
                     res += "\t%-12s\t%s\t%s\n" % (k[:-5], date, delay[idx])
                     idx += 1
-            content = result["content"]
-            content = json.loads(content)
-            res += "CONTENT"
-            res += json.dumps(content, indent=True) + "\n"
+            if "content" in result:
+                content = result["content"]
+                content = json.loads(content)
+                res += "CONTENT"
+                res += json.dumps(content, indent=True) + "\n"
     elif cmd == "task" and subcmd == "delete":
         r = requests.get(os.path.join(args.url, "task/list", args.prefix + '*'), auth=auth)
         if r.status_code != 200:
@@ -622,8 +657,7 @@ def process_request(service_list, cmd, subcmd, is_json, args, auth=None):
         res = r.text.encode("utf-8")
     if res is not None:
         return res
-    else:
-        return result
+    return result
 
 
 if __name__ == "__main__":
@@ -647,31 +681,31 @@ if __name__ == "__main__":
             LOGGER.error('missing launcher_url')
             sys.exit(1)
 
-    r = requests.get(os.path.join(ARGS.url, "service/list"), params={"minimal": True})
-    if r.status_code != 200:
-        LOGGER.error('incorrect result from \'service/list\' service: %s', r.text)
+    response = requests.get(os.path.join(ARGS.url, "service/list"), params={"minimal": True})
+    if response.status_code != 200:
+        LOGGER.error('incorrect result from \'service/list\' service: %s', response.text)
         sys.exit(1)
 
-    service_list = r.json()
+    service_list = response.json()
 
     try:
-        res = process_request(service_list, ARGS.cmd, ARGS.subcmd, ARGS.display == "JSON", ARGS)
-    except RuntimeError as err:
-        LOGGER.error(err)
+        request_res = process_request(service_list, ARGS.cmd, ARGS.subcmd, ARGS.display == "JSON", ARGS)
+    except RuntimeError as error:
+        LOGGER.error(error)
         sys.exit(1)
-    except ValueError as err:
-        LOGGER.error(err)
+    except ValueError as error:
+        LOGGER.error(error)
         sys.exit(1)
 
-    if ARGS.display == "JSON" or isinstance(res, dict):
-        print(json.dumps(res))
-    elif isinstance(res, PrettyTable):
+    if ARGS.display == "JSON" or isinstance(request_res, dict):
+        print(json.dumps(request_res))
+    elif isinstance(request_res, PrettyTable):
         if ARGS.display == "TABLE":
-            print(res)
+            print(request_res)
         elif ARGS.display == "RAW":
-            res.set_style(PLAIN_COLUMNS)
-            print(res)
+            request_res.set_style(PLAIN_COLUMNS)
+            print(request_res)
         else:
-            print(res.get_html_string())
+            print(request_res.get_html_string())
     else:
-        print(res.strip())
+        print(request_res.strip())
